@@ -1,9 +1,9 @@
 __author__ = """Yasser Mohammad"""
 __email__ = "yasserfarouk@gmail.com"
-__version__ = "0.5.5"
+__version__ = "0.7.1"
 
 from .scml2019 import *
 from .scml2020 import *
 from .oneshot import *
 
-__all__ = scml2020.__all__ + oneshot.__all__
+__all__ = scml2020.__all__ + oneshot.__all__ + ["utils", "runner"]
