@@ -13,39 +13,36 @@ Remarks:
       which is useful if you are keeping information about past negotiations and
       partner behavior.
 """
+
 from __future__ import annotations
 
 import warnings
-from abc import ABC
-from abc import abstractmethod
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Dict
-from typing import List
-from typing import Optional
+from abc import ABC, abstractmethod
+from typing import TYPE_CHECKING, Any
 
-from negmas import Contract
-from negmas import ControlledSAONegotiator
-from negmas import Entity
-from negmas import Issue
-from negmas import MechanismState
-from negmas import NegotiatorMechanismInterface
-from negmas import Outcome
-from negmas import ResponseType
-from negmas import SAOController
-from negmas import SAOResponse
-from negmas import SAOSingleAgreementController
-from negmas import SAOState
-from negmas import SAOSyncController
+from negmas import (
+    ConstUtilityFunction,
+    Contract,
+    ControlledNegotiator,
+    ControlledSAONegotiator,
+    Entity,
+    Issue,
+    NegotiatorMechanismInterface,
+    Outcome,
+    ResponseType,
+    SAOController,
+    SAOResponse,
+    SAOSingleAgreementController,
+    SAOState,
+    SAOSyncController,
+)
 from negmas.helpers import get_class
 from negmas.preferences import UtilityFunction
-from negmas.sao import SAONMI
-from negmas.sao import SAONegotiator
+from negmas.sao import SAONMI, SAONegotiator
 from negmas.situated import RunningNegotiationInfo
 
 if TYPE_CHECKING:
-    from scml.oneshot import OneShotAWI
-    from scml.oneshot import OneShotUFun
+    from scml.oneshot import OneShotAWI, OneShotUFun
 __all__ = [
     "OneShotAgent",
     "OneShotSyncAgent",
@@ -57,15 +54,8 @@ __all__ = [
 
 class OneShotAgent(SAOController, Entity, ABC):
     """
-
     Base class for all agents in the One-Shot game.
 
-    Args:
-        owner: The adapter owning the agent. You do not need to directly deal
-               with this.
-        ufun: An optional `OneShotUFun` to set for the agent.
-        name: Agent name.
-
     Remarks:
         - You can access all of the negotiators associated with the agent using
           `self.negotiators` which is a dictionary mapping the `negotiator_id` to
@@ -105,6 +95,7 @@ class OneShotAgent(SAOController, Entity, ABC):
 
             A list of negotiation information objects (`RunningNegotiationInfo`)
         """
+        assert self._owner is not None
         return self._owner.running_negotiations
 
     @property
@@ -112,6 +103,7 @@ class OneShotAgent(SAOController, Entity, ABC):
         """
         All contracts that are not yet signed.
         """
+        assert self._owner is not None
         return self._owner.unsigned_contracts
 
     def init(self):
@@ -136,11 +128,22 @@ class OneShotAgent(SAOController, Entity, ABC):
               You will not need to directly use this method in most cases.
 
         """
+        assert self._owner is not None
         return self._owner.make_ufun(add_exogenous)
 
+    def before_step(self):
+        """
+        Called at the beginning of every step.
+
+        Remarks:
+            - Use this for any proactive code  that needs to be done every
+              simulation step.
+        """
+        pass
+
     def step(self):
         """
-        Called every step.
+        Called at the end of every step.
 
         Remarks:
             - Use this for any proactive code  that needs to be done every
@@ -161,7 +164,7 @@ class OneShotAgent(SAOController, Entity, ABC):
         self.utility_function = owner.ufun
 
     @abstractmethod
-    def propose(self, negotiator_id: str, state: MechanismState) -> Outcome:
+    def propose(self, negotiator_id: str, state: SAOState) -> Outcome | None:
         """
         Proposes an offer to one of the partners.
 
@@ -173,16 +176,13 @@ class OneShotAgent(SAOController, Entity, ABC):
             an outcome to offer.
         """
 
-    def respond(
-        self, negotiator_id: str, state: MechanismState, offer: Outcome
-    ) -> ResponseType:
+    def respond(self, negotiator_id: str, state: SAOState, source=None) -> ResponseType:
         """
         Responds to an offer from one of the partners.
 
         Args:
             negotiator_id: ID of the negotiator (and partner)
             state: Mechanism state including current step
-            offer: The offer received.
 
         Returns:
             A response type which can either be reject, accept, or end negotiation.
@@ -193,6 +193,7 @@ class OneShotAgent(SAOController, Entity, ABC):
             proposed in the given state and reject otherwise
 
         """
+        offer = state.current_offer  # type: ignore
         myoffer = self.propose(negotiator_id, state)
         if myoffer == offer:
             return ResponseType.ACCEPT_OFFER
@@ -214,7 +215,7 @@ class OneShotAgent(SAOController, Entity, ABC):
         partners: list[str],
         annotation: dict[str, Any],
         mechanism: NegotiatorMechanismInterface,
-        state: MechanismState,
+        state: SAOState,
     ) -> None:
         """
         Called whenever a negotiation ends without agreement.
@@ -276,17 +277,10 @@ class OneShotAgent(SAOController, Entity, ABC):
         return self.negotiators[partner_id][0].nmi
 
 
-class OneShotSyncAgent(SAOSyncController, OneShotAgent, ABC):
+class OneShotSyncAgent(SAOSyncController, OneShotAgent, ABC):  # type: ignore
     """
     An agent that automatically accumulate offers from opponents and allows
     you to control all negotiations centrally in the `counter_all` method.
-
-    Args:
-        owner: The adapter owning the agent. You do not need to directly deal
-               with this.
-        ufun: An optional `OneShotUFun` to set for the agent.
-        name: Agent name.
-
     """
 
     def __init__(self, *args, **kwargs):
@@ -295,7 +289,7 @@ class OneShotSyncAgent(SAOSyncController, OneShotAgent, ABC):
 
     @abstractmethod
     def counter_all(
-        self, offers: dict[str, Outcome], states: dict[str, SAOState]
+        self, offers: dict[str, Outcome | None], states: dict[str, SAOState]
     ) -> dict[str, SAOResponse]:
         """Calculate a response to all offers from all negotiators
         (negotiator ID is the key).
@@ -319,7 +313,7 @@ class OneShotSyncAgent(SAOSyncController, OneShotAgent, ABC):
         """
 
     @abstractmethod
-    def first_proposals(self) -> dict[str, Outcome]:
+    def first_proposals(self) -> dict[str, Outcome | None]:
         """
         Gets a set of proposals to use for initializing the negotiation.
 
@@ -336,7 +330,7 @@ class OneShotSyncAgent(SAOSyncController, OneShotAgent, ABC):
         return [self.id] * len(contracts)
 
 
-class OneShotSingleAgreementAgent(SAOSingleAgreementController, OneShotSyncAgent):
+class OneShotSingleAgreementAgent(SAOSingleAgreementController, OneShotSyncAgent):  # type: ignore
     """
     A synchronized agent that tries to get no more than one agreement.
 
@@ -401,7 +395,9 @@ class OneShotSingleAgreementAgent(SAOSingleAgreementController, OneShotSyncAgent
         """
 
     @abstractmethod
-    def is_better(self, a: Outcome, b: Outcome, negotiator: str, state: SAOState):
+    def is_better(
+        self, a: Outcome | None, b: Outcome | None, negotiator: str, state: SAOState
+    ) -> bool:
         """Compares two outcomes of the same negotiation
 
         Args:
@@ -415,11 +411,11 @@ class OneShotSingleAgreementAgent(SAOSingleAgreementController, OneShotSyncAgent
         """
 
 
-class EndingNegotiator(SAONegotiator):
+class EndingNegotiator(SAONegotiator, ControlledNegotiator):
     def propose(self, state):
         return None
 
-    def respond(self, state, offer):
+    def respond(self, state, source=None):
         return ResponseType.END_NEGOTIATION
 
 
@@ -527,14 +523,14 @@ class OneShotIndNegotiatorsAgent(OneShotAgent):
 
     def _get_ufuns(self):
         """
-        Internam method that makes sure the reservation value is set to a
+        Internal method that makes sure the reservation value is set to a
         meaningful value and that the ufun is normalized if needed
         """
         ufuns = self.generate_ufuns()
         if not self._normalize and not self._set_reservation:
             return ufuns
         if not self.awi:
-            raise ValueError(f"AWI is not found!!!")
+            raise ValueError("AWI is not found!!!")
         for partner_id, u in ufuns.items():
             if self.awi.is_system(partner_id):
                 continue
@@ -543,7 +539,7 @@ class OneShotIndNegotiatorsAgent(OneShotAgent):
                 if partner_id in self.awi.my_suppliers
                 else self.awi.current_output_issues
             )
-            mn, mx = self._urange(u, issues)
+            mn, mx = self._urange(u, tuple(issues))
             if self._normalize:
                 u = self._unorm(u, mn, mx)
                 if u is None:
@@ -558,7 +554,7 @@ class OneShotIndNegotiatorsAgent(OneShotAgent):
                 u.reserved_value = mn - 1e-5
             u.reserved_value = u.reserved_value / mx
             if u.reserved_value > mx:
-                ufuns[partner_id] = None
+                ufuns[partner_id] = ConstUtilityFunction(0.0, reserved_value=0.0)
         return ufuns
 
     def init(self):
@@ -572,9 +568,9 @@ class OneShotIndNegotiatorsAgent(OneShotAgent):
     def make_negotiator(
         self,
         negotiator_type=None,
-        name: str = None,
+        name: str | None = None,
         **kwargs,
-    ):
+    ) -> ControlledSAONegotiator:
         """
         Creates a negotiator but does not add it to the controller. Call
         `add_negotiator` to add it.
@@ -593,14 +589,16 @@ class OneShotIndNegotiatorsAgent(OneShotAgent):
             is to be thrown.
 
         """
+        if name is None:
+            return EndingNegotiator()  # type: ignore
         ufun = self._ufuns[name]
         if ufun is None:
-            return EndingNegotiator()
+            return EndingNegotiator()  # type: ignore
         negotiator = self.generate_negotiator(name)
         negotiator.id = name
         negotiator.name = name
         negotiator.ufun = ufun
-        return negotiator
+        return negotiator  # type: ignore
 
     def propose(self, negotiator_id, state):
         raise ValueError(
