@@ -2,15 +2,15 @@ import itertools
 import random
 from collections import defaultdict
 
-from negmas import ResponseType
-from negmas import SAOResponse
+from negmas import Outcome, ResponseType, SAOResponse
 
-from scml.oneshot.agent import OneShotAgent
-from scml.oneshot.agent import OneShotSingleAgreementAgent
-from scml.oneshot.agent import OneShotSyncAgent
-from scml.scml2020.common import QUANTITY
-from scml.scml2020.common import TIME
-from scml.scml2020.common import UNIT_PRICE
+from scml.oneshot.agent import (
+    OneShotAgent,
+    OneShotSingleAgreementAgent,
+    OneShotSyncAgent,
+)
+from scml.oneshot.common import QUANTITY, TIME, UNIT_PRICE
+from scml.oneshot.ufun import OneShotUFun
 
 __all__ = ["GreedyOneShotAgent", "GreedySyncAgent", "GreedySingleAgreementAgent"]
 
@@ -47,7 +47,7 @@ class GreedyOneShotAgent(OneShotAgent):
         opp_price_slack=None,
         opp_acc_price_slack=None,
         range_slack=None,
-        **kwargs
+        **kwargs,
     ):
         super().__init__(*args, **kwargs)
         if concession_exponent is None:
@@ -102,7 +102,7 @@ class GreedyOneShotAgent(OneShotAgent):
                 up, self._best_opp_acc_buying[partner]
             )
 
-    def propose(self, negotiator_id: str, state):
+    def propose(self, negotiator_id: str, state, source=None) -> Outcome | None:
         # find the absolute best offer for me. This will most likely has an
         # unrealistic price
         offer = self.best_offer(negotiator_id)
@@ -116,7 +116,9 @@ class GreedyOneShotAgent(OneShotAgent):
         offer[UNIT_PRICE] = self._find_good_price(self.get_nmi(negotiator_id), state)
         return tuple(offer)
 
-    def respond(self, negotiator_id, state, offer):
+    def respond(self, negotiator_id, state, source=None) -> ResponseType:
+        offer = state.current_offer  # type: ignore
+        assert offer is not None
         # find the quantity I still need and end negotiation if I need nothing more
         my_needs = self._needed(negotiator_id)
         if my_needs <= 0:
@@ -258,7 +260,7 @@ class GreedyOneShotAgent(OneShotAgent):
         return ((n_steps - step - 1) / (n_steps - 1)) ** self._e
 
 
-class GreedySyncAgent(OneShotSyncAgent, GreedyOneShotAgent):
+class GreedySyncAgent(OneShotSyncAgent, GreedyOneShotAgent):  # type: ignore
     """A greedy agent based on OneShotSyncAgent"""
 
     def __init__(self, *args, threshold=None, **kwargs):
@@ -266,11 +268,12 @@ class GreedySyncAgent(OneShotSyncAgent, GreedyOneShotAgent):
         if threshold is None:
             threshold = random.random() * 0.2 + 0.2
         self._threshold = threshold
+        self.ufun: OneShotUFun  # type: ignore
 
     def before_step(self):
         super().before_step()
-        self.ufun.best = self.ufun.find_limit(True)
-        self.ufun.worst = self.ufun.find_limit(False)
+        self.ufun.find_limit(True)
+        self.ufun.find_limit(False)
 
     def first_proposals(self):
         """Decide a first proposal on every negotiation.
@@ -282,7 +285,7 @@ class GreedySyncAgent(OneShotSyncAgent, GreedyOneShotAgent):
             )
         )
 
-    def counter_all(self, offers, states):
+    def counter_all(self, offers, states) -> dict:
         """Respond to a set of offers given the negotiation state of each."""
 
         if self.ufun.max_utility < 0:
@@ -356,16 +359,23 @@ class GreedySyncAgent(OneShotSyncAgent, GreedyOneShotAgent):
             return 0, self.awi.current_exogenous_input_quantity - self._sales
         return self.awi.current_exogenous_output_quantity - self._supplies, 0
 
+    def propose(self, negotiator_id, state):  # type: ignore
+        return OneShotSyncAgent.propose(self, negotiator_id, state)
+
+    def respond(self, negotiator_id, state, source=""):  # type: ignore
+        return OneShotSyncAgent.respond(self, negotiator_id, state, source)
+
 
 class GreedySingleAgreementAgent(OneShotSingleAgreementAgent):
     """A greedy agent based on `OneShotSingleAgreementAgent`"""
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self.ufun: OneShotUFun  # type: ignore
 
     def before_step(self):
-        self.ufun.best = self.ufun.find_limit(True)
-        self.ufun.worst = self.ufun.find_limit(False)
+        self.ufun.find_limit(True)
+        self.ufun.find_limit(False)
 
     def is_acceptable(self, offer, source, state) -> bool:
         mx, mn = self.ufun.max_utility, self.ufun.min_utility
