@@ -1,32 +1,17 @@
 from __future__ import annotations
 
 from collections import namedtuple
-from copy import copy
-from functools import lru_cache
-from typing import Iterable
-from typing import List
-from typing import Literal
-from typing import Optional
-from typing import Tuple
-from typing import Union
-from typing import overload
+from functools import cache
+from typing import Iterable, Literal, overload
 
+from attr import define
 from negmas import Contract
-from negmas.outcomes import Issue
-from negmas.outcomes import Outcome
-from negmas.outcomes import OutcomeSpace
-from negmas.outcomes import make_issue
-from negmas.outcomes import make_os
-from negmas.preferences import StationaryMixin
-from negmas.preferences import UtilityFunction
+from negmas.outcomes import Issue, Outcome, OutcomeSpace, make_issue, make_os
+from negmas.preferences import StationaryMixin, UtilityFunction
 
-from scml.scml2020.common import is_system_agent
+from .common import QUANTITY, TIME, UNIT_PRICE, is_system_agent
 
-from .common import QUANTITY
-from .common import TIME
-from .common import UNIT_PRICE
-
-__all__ = ["OneShotUFun", "UFunLimit"]
+__all__ = ["OneShotUFun", "UFunLimit", "UtilityInfo"]
 
 UFunLimit = namedtuple(
     "UFunLimit",
@@ -40,13 +25,28 @@ UFunLimit = namedtuple(
         "exogenous_input_price",
         "exogenous_output_quantity",
         "exogenous_output_price",
+        "inventory_input",
+        "inventory_output",
         "producible",
     ],
 )
 """Information about one utility limit (either highest or lowest). See `OnShotUFun.find_limit` for details."""
 
 
-class OneShotUFun(StationaryMixin, UtilityFunction):
+@define
+class UtilityInfo:
+    producible: int
+    total_input: int
+    total_output: int
+    shortfall_quantity: int
+    shortfall_penalty: float
+    remaining_quantity: int
+    disposal_cost: float
+    storage_cost: float
+    utility: float
+
+
+class OneShotUFun(StationaryMixin, UtilityFunction):  # type: ignore
     """
     Calculates the utility function of a list of contracts or offers.
 
@@ -105,12 +105,18 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         output_agent: bool,
         production_cost: float,
         disposal_cost: float,
+        storage_cost: float,
         shortfall_penalty: float,
         input_penalty_scale: float | None,
         output_penalty_scale: float | None,
+        storage_penalty_scale: float | None,
         n_input_negs: int,
         n_output_negs: int,
         current_step: int,
+        agent_id: str | None,
+        time_range: tuple[int, int],
+        inventory_in: int = 0,
+        inventory_out: int = 0,
         input_qrange: tuple[int, int] = (0, 0),
         input_prange: tuple[int, int] = (0, 0),
         output_qrange: tuple[int, int] = (0, 0),
@@ -119,23 +125,49 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         n_lines: int = 10,
         normalized: bool = False,
         current_balance: int | float = float("inf"),
+        suppliers: set[str] = set(),
+        consumers: set[str] = set(),
+        perishable=True,
         **kwargs,
     ):
         super().__init__(**kwargs)
+        self.agent_id = agent_id
+        self.time_range = time_range
+        self._best, self._worst = None, None
+        self.suppliers = suppliers
+        self.consumers = consumers
         self.current_balance = current_balance
         self.normalized = normalized
         self.input_penalty_scale = input_penalty_scale
+        self.storage_penalty_scale = storage_penalty_scale
         self.output_penalty_scale = output_penalty_scale
         self.current_step = current_step
         self.ex_pin, self.ex_pout = ex_pin, ex_pout
         self.ex_qin, self.ex_qout = ex_qin, ex_qout
+        self.inventory_in = inventory_in
+        self.inventory_out = inventory_out
         self.n_input_negs = n_input_negs
+        self.perishable = perishable
         self.n_output_negs = n_output_negs
         self.input_qrange, self.input_prange = input_qrange, input_prange
         self.output_qrange, self.output_prange = output_qrange, output_prange
-        self.production_cost, self.disposal_cost, self.shortfall_penalty = (
+        if production_cost is None:
+            production_cost = 0.0
+        if disposal_cost is None:
+            disposal_cost = 0.0
+        if storage_cost is None:
+            storage_cost = 0.0
+        if shortfall_penalty is None:
+            shortfall_penalty = 0.0
+        (
+            self.production_cost,
+            self.disposal_cost,
+            self.storage_cost,
+            self.shortfall_penalty,
+        ) = (
             production_cost,
             disposal_cost,
+            storage_cost,
             shortfall_penalty,
         )
         self.input_agent, self.output_agent = input_agent, output_agent
@@ -151,11 +183,11 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         else:
             self.output_product = None
         if self.normalized:
-            self.best = self.find_limit(True, None, None)
-            self.worst = self.find_limit(False, None, None)
+            self._best = self.find_limit(True, None, None)
+            self._worst = self.find_limit(False, None, None)
         else:
-            self.best = UFunLimit(*tuple([None] * 10))
-            self.worst = UFunLimit(*tuple([None] * 10))
+            self._best = UFunLimit(*tuple([None] * 12))
+            self._worst = UFunLimit(*tuple([None] * 12))
         if self.input_agent or self.output_agent:
             # if this is an edge agent, all negotiations will be on the same product so we can define its outcome-space
             qrange = self.input_qrange if self.output_agent else self.output_qrange
@@ -163,7 +195,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             self.outcome_space = make_os(
                 [
                     make_issue(qrange, name="quantity"),
-                    make_issue((self.current_step, self.current_step), name="time"),
+                    make_issue(time_range, name="time"),
                     make_issue(prange, name="unit_price"),
                 ]
             )
@@ -173,7 +205,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
                 make_os(
                     [
                         make_issue(qrange, name="quantity"),
-                        make_issue((self.current_step, self.current_step), name="time"),
+                        make_issue(time_range, name="time"),
                         make_issue(prange, name="unit_price"),
                     ]
                 )
@@ -184,11 +216,61 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             ]
         # slightly bias toward agreements
         self.reserved_value = self.from_contracts([], ignore_exogenous=False) - 1e-3
+        self._signed_agreements: list[tuple[int, int, int]] = []
+        self._signed_is_output: list[bool] = []
+        self._registered_sale_failures: set[str] = set()
+        self._registered_supply_failures: set[str] = set()
+        if perishable:
+            assert (
+                self.storage_cost == 0
+            ), f"Perishable ufun but {self.storage_cost=} ({self.disposal_cost})"
+        else:
+            assert (
+                self.disposal_cost == 0
+            ), f"Non-perishable ufun but {self.disposal_cost=} ({self.storage_cost})"
+
+    @property
+    def best_option(self) -> UFunLimit:
+        """Best possible options"""
+        if self._best is None:
+            self._best = self.find_limit(True)
+        return self._best
+
+    @property
+    def worst_option(self) -> UFunLimit:
+        """Best possible options"""
+        if self._worst is None:
+            self._worst = self.find_limit(False)
+        return self._worst
+
+    def register_supply_failure(self, supplier_id: str):
+        self.find_limit_brute_force.cache_clear()
+        self._registered_supply_failures.add(supplier_id)
+
+    def register_sale_failure(self, consumer_id: str):
+        self.find_limit_brute_force.cache_clear()
+        self._registered_sale_failures.add(consumer_id)
+
+    def register_sale(self, q: int, p: int, t: int):
+        """Registers a sale to be considered when calculating utilities"""
+        if t != self.current_step:
+            return
+        self.find_limit_brute_force.cache_clear()
+        self._signed_agreements.append((q, t if t >= 0 else self.current_step, p))
+        self._signed_is_output.append(True)
+
+    def register_supply(self, q: int, p: int, t: int):
+        """Registers a supply to be considered when calculating utilities"""
+        if t != self.current_step:
+            return
+        self.find_limit_brute_force.cache_clear()
+        self._signed_agreements.append((q, t if t >= 0 else self.current_step, p))
+        self._signed_is_output.append(False)
 
     def xml(self, issues) -> str:
-        raise NotImplementedError("Cannot convert the ufun to xml")
+        raise NotImplementedError(f"Cannot convert the ufun to xml: {issues}")
 
-    def eval(self, offer) -> float:
+    def eval(self, offer: tuple[int, int, int] | None) -> float:  # type: ignore
         """
         Calculates the utility function given a single contract.
 
@@ -198,11 +280,34 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         """
         if not self.input_agent and not self.output_agent:
             return float("-inf")
-        return self.from_offers((tuple(offer) if offer else None,), (self.input_agent,))
+        if offer is not None:
+            offer = tuple(offer)  # type: ignore
+        return self.from_offers((offer if offer else None,), (self.input_agent,))
+
+    @overload
+    def from_contracts(
+        self,
+        contracts: Iterable[Contract],
+        return_info: Literal[False] = False,
+        ignore_exogenous=True,
+    ) -> float:
+        ...
+
+    @overload
+    def from_contracts(
+        self,
+        contracts: Iterable[Contract],
+        return_info: Literal[True],
+        ignore_exogenous=True,
+    ) -> UtilityInfo:
+        ...
 
     def from_contracts(
-        self, contracts: Iterable[Contract], ignore_exogenous=True
-    ) -> float:
+        self,
+        contracts: Iterable[Contract],
+        return_info: bool = False,
+        ignore_exogenous=True,
+    ) -> float | UtilityInfo:
         """
         Calculates the utility function given a list of contracts
 
@@ -225,13 +330,29 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         for c in contracts:
             if c.signed_at < 0:
                 continue
+            if c.nullified_at >= 0:
+                continue
             if ignore_exogenous and any(is_system_agent(_) for _ in c.partners):
                 continue
             product = c.annotation["product"]
             is_output = product == output_product
+            assert (
+                c.annotation["buyer"] != c.annotation["seller"]
+            ), f"{self.agent_id=}: Buyer == Seller == {c.annotation['buyer']}"
+            assert (is_output and c.annotation["buyer"] != self.agent_id) or (
+                not is_output and c.annotation["seller"] != self.agent_id
+            ), (
+                f"{self.agent_id=}: Got contract in which I am either buying "
+                f"my output product or selling my input product: {self.input_product=}"
+                f", {self.output_product=}\n{c}"
+            )
             outputs.append(is_output)
             offers.append(self.outcome_as_tuple(c.agreement))
-        return self.from_offers(tuple(offers), tuple(outputs))
+        return self.from_offers(  # type: ignore
+            tuple(offers),
+            tuple(outputs),
+            return_info=return_info,  # type: ignore
+        )
 
     @staticmethod
     def outcome_as_tuple(offer):
@@ -246,28 +367,33 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
     @overload
     def from_offers(
         self,
-        offers: tuple[tuple[int, int, int], ...],
-        outputs: tuple[bool],
-        return_producible: Literal[False] = False,
+        offers: tuple[tuple[int, int, int | float] | None, ...]
+        | dict[str, tuple[int, int, int] | None],
+        outputs: tuple[bool, ...] | None = None,
+        return_info: Literal[False] = False,
+        ignore_signed_contracts: bool = True,
     ) -> float:
         ...
 
     @overload
     def from_offers(
         self,
-        offers: tuple[tuple[int, int, int], ...],
-        outputs: tuple[bool],
-        return_producible: Literal[True],
-    ) -> tuple[float, float]:
+        offers: tuple[tuple[int, int, int | float] | None, ...]
+        | dict[str, tuple[int, int, int] | None],
+        outputs: tuple[bool, ...] | None,
+        return_info: Literal[True],
+        ignore_signed_contracts: bool = True,
+    ) -> UtilityInfo:
         ...
 
-    @lru_cache
     def from_offers(
         self,
-        offers: tuple[tuple[int, int, int], ...],
-        outputs: tuple[bool],
-        return_producible=False,
-    ) -> float | tuple[float, int]:
+        offers: tuple[tuple[int, int, int | float] | None, ...]
+        | dict[str, tuple[int, int, int] | None],
+        outputs: tuple[bool, ...] | None = None,
+        return_info: bool = False,
+        ignore_signed_contracts: bool = True,
+    ) -> float | UtilityInfo:
         """
         Calculates the utility value given a list of offers and whether each
         offer is for output or not (= input).
@@ -279,11 +405,42 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             outputs: An iterable of the same length as offers of booleans
                      specifying for each offer whether it is an offer for buying
                      the agent's output product.
-            return_producible: If true, the producible quantity will be returned
+            return_info: If true, detailed utility information is returned as Utility Info
+            ignore_signed_contracts: If true, ignores the registered signed contracts.
+                                     This means that only exogenous contracts and offers
+                                     will be used in evaluating the utility.
         Remarks:
             - This method takes into account the exogenous contract information
               passed when constructing the ufun.
+            - You can pass a dictionary mapping partner ID to an offer and the system
+              will use the correct value for the corresponding outputs array.
         """
+        if isinstance(offers, dict):
+            partners: list[str] = list(offers.keys())
+            offers = tuple(offers.values())
+            outputs = tuple(p in self.consumers for p in partners)
+            # assert all(
+            #     (p in self.consumers and not p in self.suppliers)
+            #     or (p in self.suppliers and not p in self.consumers)
+            #     for p in partners
+            # )
+            return self.from_offers(
+                offers,
+                outputs,
+                return_info=return_info,  # type: ignore
+                ignore_signed_contracts=ignore_signed_contracts,
+            )
+        # copy inputs because we are going to modify them.
+        offers = list(offers)  # type: ignore
+        if outputs is None:
+            if self.input_agent:
+                outputs = tuple([True] * len(offers))
+            elif self.output_agent:
+                outputs = tuple([False] * len(offers))
+            else:
+                raise RuntimeError(
+                    "You cannot pass outputs=None if the agent is neither a first or last level agent"
+                )
 
         def order(x):
             """A helper function to order contracts in the following fashion:
@@ -291,22 +448,36 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             2. output contracts are ordered from highest price to cheapest.
             3. The relative order of input and output contracts is indeterminate.
             """
-            offer, is_output, is_exogenous = x
+            offer, is_output, _ = x
             # if is_exogenous and self.force_exogenous:
             #     return float("-inf")
             return -offer[UNIT_PRICE] if is_output else offer[UNIT_PRICE]
 
         # copy inputs because we are going to modify them.
-        offers, outputs = list(offers), list(outputs)
+        outputs = list(outputs)  # type: ignore
+        # add registered sales and supplies if needed
+        if not ignore_signed_contracts and self._signed_agreements:
+            offers += self._signed_agreements  # type: ignore
+            outputs += self._signed_is_output  # type: ignore
         # indicate that all inputs are not exogenous and that we are adding two
         # exogenous contracts after them.
         exogenous = [False] * len(offers) + [True, True]
         # add exogenous contracts as offers one for input and another for output
-        offers += [
-            (self.ex_qin, 0, self.ex_pin / self.ex_qin if self.ex_qin else 0),
-            (self.ex_qout, 0, self.ex_pout / self.ex_qout if self.ex_qout else 0),
+        invin = self.ex_qin + self.inventory_in
+        invout = self.ex_qout + self.inventory_out
+        offers += [  # type: ignore
+            (
+                invin,
+                self.current_step,
+                self.ex_pin / invin if invin else 0,
+            ),
+            (
+                invout,
+                self.current_step,
+                self.ex_pout / invout if invout else 0,
+            ),
         ]
-        outputs += [False, True]
+        outputs += [False, True]  # type: ignore
         # initialize some variables
         qin, qout, pin, pout = 0, 0, 0, 0
         qin_bar, going_bankrupt = 0, self.current_balance < 0
@@ -315,15 +486,22 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         output_offers = []
         # sort contracts in the optimal order of execution: from cheapest when
         # buying and from the most expensive when selling. See `order` above.
-        sorted_offers = sorted(zip(offers, outputs, exogenous), key=order)
+        assert outputs is not None
+        sorted_offers = sorted(
+            list(zip(offers, outputs, exogenous, strict=True)),
+            key=order,
+        )
 
         # we calculate the total quantity we are are required to pay for `qin` and
         # the associated amount of money we are going to pay `pin`. Moreover,
         # we calculate the total quantity we can actually buy given our limited
         # money balance (`qin_bar`).
-        for offer, is_output, is_exogenous in sorted_offers:
+        for offer, is_output, is_exogenous in sorted_offers:  # type: ignore
             if not offer:
                 continue
+            # ignore any offers that are not about this time
+            if offer[TIME] != self.current_step:
+                continue
             offer: tuple[int, int, int]
             if is_output:
                 output_offers.append((offer, is_exogenous))
@@ -348,10 +526,10 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         n_lines = self.n_lines
         producible = min(qin_bar, n_lines)
 
-        # No need to this test now because we test for the ability to produce with
+        # No need to do this test now because we test for the ability to produce with
         # the ability to buy items. The factory buys cheaper items and produces them
         # before attempting more expensive ones. This may or may not be optimal but
-        # who cars. It is consistent that it is all that matters.
+        # who cares. It is consistent and that is all that matters.
         # # if we do not have enough money to pay for production in full, we limit
         # # the producible quantity to what we can actually produce
         # if (
@@ -366,7 +544,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         for offer, is_exogenous in output_offers:
             if not done_selling:
                 if qout + offer[QUANTITY] >= producible:
-                    assert producible >= qout, f"producible {producible}, qout {qout}"
+                    assert producible >= qout, f"{producible=}, {qout=}"
                     can_sell = producible - qout
                     done_selling = True
                 else:
@@ -387,22 +565,46 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         output_penalty = self.output_penalty_scale
         if output_penalty is None:
             output_penalty = pout / qout if qout else 0
-        output_penalty *= self.shortfall_penalty * max(0, qout - producible)
-        input_penalty = self.input_penalty_scale
-        if input_penalty is None:
-            input_penalty = pin / qin if qin else 0
-        input_penalty *= self.disposal_cost * max(0, qin - producible)
+        shortfall = max(0, qout - producible)
+        output_penalty *= self.shortfall_penalty * shortfall
+        disposal_cost = self.input_penalty_scale
+        if disposal_cost is None:
+            disposal_cost = pin / qin if qin else 0
+        disposal_cost *= self.disposal_cost * max(0, qin - producible)
+        storage_cost = self.storage_penalty_scale
+        if storage_cost is None:
+            storage_cost = pin / qin if qin else 0
+        remainingq = max(0, qin - producible)
+        storage_cost *= self.storage_cost * remainingq
 
         # call a helper method giving it the total quantity and money in and out.
         u = self.from_aggregates(
-            qin, qout, producible, pin, pout_bar, input_penalty, output_penalty
+            qin,
+            qout,
+            producible,
+            pin,
+            pout_bar,
+            disposal_cost,
+            output_penalty,
+            storage_cost,
         )
-        if return_producible:
+        if return_info:
             # the real producible quantity is the minimum of what we can produce
             # given supplies and production capacity and what we can sell.
-            return u, producible
+            return UtilityInfo(
+                utility=u,
+                remaining_quantity=remainingq,
+                storage_cost=storage_cost,
+                disposal_cost=disposal_cost,
+                producible=producible,
+                total_input=qin,
+                total_output=qout,
+                shortfall_penalty=output_penalty,
+                shortfall_quantity=shortfall,
+            )
         return u
 
+    @cache
     def from_aggregates(
         self,
         qin: int,
@@ -410,8 +612,9 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         qout_sold: int,
         pin: int,
         pout: int,
-        input_penalty,
-        output_penalty,
+        input_penalty: float,
+        output_penalty: float,
+        storage_penalty: float,
     ) -> float:
         """
         Calculates the utility from aggregates of input/output quantity/prices
@@ -426,6 +629,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             pout: Output total price (i.e. unit price * qin).
             input_penalty: total disposal cost
             output_penalty: total shortfall penalty
+            storage_penalty: total storage penalty
 
         Remarks:
             - Most likely, you do not need to directly call this method. Consider
@@ -458,11 +662,14 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         #    did not. The only reason you cannot sell something is if you cannot
         #    produce it. That is why the shortfall penalty is multiplied by the
         #    difference between what you should have sold and the produced amount.
+        # 3. You pay storage penalty for anything that remains in your storage at the end.
+        # You can either have disposal penalty or storage penalty not both
         u = (
             pout
             - pin
             - self.production_cost * produced
             - input_penalty
+            - storage_penalty
             - output_penalty
         )
         if not self.normalized:
@@ -475,34 +682,34 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
 
     def breach_level(self, qin: int = 0, qout: int = 0):
         """Calculates the breach level that would result from a given quantities"""
-        qin += self.ex_qin
+        qin += self.ex_qin + self.inventory_in
         qin = min(qin, self.n_lines)
-        qout += self.ex_qout
+        qout += self.ex_qout + self.inventory_out
         return 0 if qin >= qout else (qout - qin) / qout
 
     def is_breach(self, qin: int = 0, qout: int = 0):
         """Whether the given quantities would lead to a breach."""
-        qin += self.ex_qin
-        qout += self.ex_qout
+        qin += self.ex_qin + self.inventory_in
+        qout += self.ex_qout + self.inventory_out
         return qout > min(qin, self.n_lines)
 
     @property
     def max_utility(self):
         """The maximum possible utility value"""
-        if self.best is None:
-            self.best = self.find_limit(True)
-        return self.best.utility
+        if self._best is None:
+            self._best = self.find_limit(True)
+        return self._best.utility
 
     @property
     def min_utility(self):
         """The minimum possible utility value"""
-        if self.worst is None:
-            self.worst = self.find_limit(False)
-        return self.worst.utility
+        if self._worst is None:
+            self._worst = self.find_limit(False)
+        return self._worst.utility
 
     def minmax(self, *args, **kwargs) -> tuple[float, float]:
         worst, best = self.extreme_outcomes(*args, **kwargs)
-        return self(worst), self(best)
+        return (self(worst), self(best))
 
     def extreme_outcomes(
         self,
@@ -511,6 +718,8 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         outcomes: Iterable[Outcome] | None = None,
         max_cardinality=1000,
     ) -> tuple[Outcome, Outcome]:
+        assert outcome_space is None and issues is None and outcomes is None
+        _ = max_cardinality
         product = (
             self.output_product
             if self.input_agent
@@ -545,8 +754,8 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
     def utility_range(
         self,
         outcome_space: OutcomeSpace | None = None,
-        issues: list[Issue] = None,
-        outcomes: list[Outcome] = None,
+        issues: list[Issue] | None = None,
+        outcomes: list[Outcome] | None = None,
         return_outcomes=False,
         max_n_outcomes=1000,
     ) -> tuple[float, float] | tuple[float, float, Outcome, Outcome]:
@@ -605,6 +814,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         secured_input_unit_price=0.0,
         secured_output_quantity=0,
         secured_output_unit_price=0.0,
+        ignore_signed_contracts: bool = True,
     ) -> UFunLimit:
         """
         Finds either the maximum or the minimum of the ufun.
@@ -619,6 +829,9 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
              secured_output_quantity: A quantity that MUST be sold.
              secured_output_unit_price: The (average) unit price of the quantity
                                         that MUST be sold.
+             ignore_signed_contracts: If True all signed contracts will be ignored.
+                                      Use secured_* to pass this information if you need
+                                      to in this case.
         Remarks:
             - You can use the `secured_*` arguments and control over the number
               of negotiations to consider to find the utility limits **given**
@@ -641,24 +854,64 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             secured_input_unit_price,
             secured_output_quantity,
             secured_output_unit_price,
+            ignore_signed_contracts=ignore_signed_contracts,
         )
         actual_util = self.from_offers(
             (
-                (result.output_quantity, 0, result.output_price),
-                (result.input_quantity, 0, result.input_price),
+                (result.output_quantity, self.current_step, result.output_price),
+                (result.input_quantity, self.current_step, result.input_price),
             ),
             (True, False),
+            ignore_signed_contracts=ignore_signed_contracts,
         )
         assert (
             abs(result.utility - actual_util) < 1e-2
-        ), f"UFunLimit with utility {result.utility} != actual utility {actual_util} of the outcome in it!!"
+        ), f"UFunLimit with utility {result.utility} != actual utility {actual_util} of the outcome in it!!\n{result}"
         if set_best:
-            self.best = result
+            self._best = result
         elif set_worst:
-            self.worst = result
+            self._worst = result
         return result
 
-    @lru_cache
+    def best(self) -> Outcome:
+        if self._best is None:
+            self._best = self.find_limit(True)
+        if not self._best.output_quantity:
+            return (
+                self._best.input_quantity,
+                self.current_step,
+                self._best.input_price,
+            )
+        if not self._best.input_quantity:
+            return (
+                self._best.output_quantity,
+                self.current_step,
+                self._best.output_price,
+            )
+        raise ValueError(
+            f"We need to buy and sell which means there is single contract that makes sense {self._best=}."
+        )
+
+    def worst(self) -> Outcome:
+        if self._worst is None:
+            self._worst = self.find_limit(False)
+        if not self._worst.output_quantity:
+            return (
+                self._worst.input_quantity,
+                self.current_step,
+                self._worst.input_price,
+            )
+        if not self._worst.input_quantity:
+            return (
+                self._worst.output_quantity,
+                self.current_step,
+                self._worst.output_price,
+            )
+        raise ValueError(
+            f"We need to buy and sell which means there is single contract that makes sense {self._worst=}."
+        )
+
+    @cache
     def find_limit_brute_force(
         self,
         best,
@@ -668,6 +921,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         secured_input_unit_price=0.0,
         secured_output_quantity=0,
         secured_output_unit_price=0.0,
+        ignore_signed_contracts=True,
     ) -> UFunLimit:
         """
         Finds either the maximum and the minimum of the ufun.
@@ -686,6 +940,10 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             - You can use the `secured_*` arguments and control over the number
               of negotiations to consider to find the utility limits **given**
               some already concluded and signed contracts
+            - Note that this function CANNOT take into account the sales or supplies
+              already signed (and registered via `register_sale` and/or `register_supply`).
+              You MUST pass the quantities and prices for signed contracts through the secured_*
+              parameters.
 
         Returns:
             worst and best outcome information in the form of `UFunLimit` tuple.
@@ -693,8 +951,35 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         """
         if n_input_negs is None:
             n_input_negs = self.n_input_negs
+            if not ignore_signed_contracts:
+                n_input_negs -= sum(int(_) for _ in self._signed_is_output if not _)
+                n_input_negs -= len(self._registered_supply_failures)
+                assert n_input_negs >= 0, f"{n_input_negs=} cannot be negative"
+
         if n_output_negs is None:
             n_output_negs = self.n_output_negs
+            if not ignore_signed_contracts:
+                n_output_negs -= sum(int(_) for _ in self._signed_is_output if _)
+                n_output_negs -= len(self._registered_sale_failures)
+                assert n_output_negs >= 0, f"{n_output_negs=} cannot be negative"
+
+        if not ignore_signed_contracts:
+            sales = [
+                c for c, o in zip(self._signed_agreements, self._signed_is_output) if o
+            ]
+            supplies = [
+                c
+                for c, o in zip(self._signed_agreements, self._signed_is_output)
+                if not o
+            ]
+            secured_input_quantity = sum(_[0] for _ in supplies)
+            secured_input_unit_price = sum(_[-1] * _[0] for _ in supplies) / (
+                secured_input_quantity if secured_input_quantity else 1
+            )
+            secured_output_quantity = sum(_[0] for _ in sales)
+            secured_output_unit_price = sum(_[-1] * _[0] for _ in sales) / (
+                secured_output_quantity if secured_output_quantity else 1
+            )
         imax = n_input_negs * self.input_qrange[1] + 1
         omax = n_output_negs * self.output_qrange[1] + 1
 
@@ -705,11 +990,26 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
         limit_p, limit_p = 0, 0
         for i in range(imax):
             for o in range(omax):
-                u, p = self.from_offers(
-                    ((i, 0, ip), (o, 0, op)),
-                    (False, True),
-                    return_producible=True,
+                info = self.from_offers(
+                    (
+                        (i, self.current_step, ip),
+                        (o, self.current_step, op),
+                        (
+                            secured_input_quantity,
+                            self.current_step,
+                            secured_input_unit_price,
+                        ),
+                        (
+                            secured_output_quantity,
+                            self.current_step,
+                            secured_output_unit_price,
+                        ),
+                    ),
+                    (False, True, False, True),
+                    return_info=True,
+                    ignore_signed_contracts=True,
                 )
+                u, p = info.utility, info.producible
                 if (best and u >= limit_u) or (not best and u <= limit_u):
                     limit_io, limit_u, limit_p = (
                         (i, ip, o, op),
@@ -718,6 +1018,7 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
                     )
         # this method cannot find the exogenous quantities at the limit found
         # if force_exogenous was false and will return None for them.
+        assert limit_io is not None
         return UFunLimit(
             utility=limit_u,
             input_quantity=limit_io[0],
@@ -728,6 +1029,8 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
             exogenous_output_price=self.ex_pout / self.ex_qout if self.ex_qout else 0,
             exogenous_input_quantity=self.ex_qin if self.force_exogenous else None,
             exogenous_output_quantity=self.ex_qout if self.force_exogenous else None,
+            inventory_input=self.inventory_in,
+            inventory_output=self.inventory_out,
             producible=limit_p,
         )
 
@@ -749,16 +1052,21 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
               always return `True` because in these cases we do not know what the
               the unit price for the output so there is nothing to compare with.
         """
+        if not self.perishable:
+            return True
+        assert self.inventory_in <= 0, f"A perishable case but {self.inventory_in=}"
+        assert self.inventory_out <= 0, f"A perishable case but {self.inventory_out=}"
         # can reject a price only if we know the output unit price
         # (i.e. we have an output agent)
         if not self.output_agent:
             return True
+        total_out = self.ex_qout
         # If we are not selling, we should not buy
-        if self.ex_qout < 1:
+        if total_out < 1:
             return False
         # do not buy at this price if it is **guaranteed** to lead to a loss
         return (
-            unit_price + self.production_cost - self.ex_pout // self.ex_qout
+            unit_price + self.production_cost - self.ex_pout // total_out
         ) < self.shortfall_penalty
 
     def ok_to_sell_at(self, unit_price: float) -> bool:
@@ -785,6 +1093,10 @@ class OneShotUFun(StationaryMixin, UtilityFunction):
               always return `True` because in these cases we do not know what the
               the unit price for the input so there is nothing to compare with.
         """
+        if not self.perishable:
+            return True
+        assert self.inventory_in <= 0, f"A perishable case but {self.inventory_in=}"
+        assert self.inventory_out <= 0, f"A perishable case but {self.inventory_out=}"
         # can reject a price only if we know the input unit price
         # (i.e. we have an input agent)
         if not self.input_agent:
