@@ -3,27 +3,23 @@ import itertools
 import math
 from abc import abstractmethod
 from collections import defaultdict
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Dict
-from typing import List
-from typing import Optional
+from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
 from negmas import UtilityFunction
 from negmas.common import NegotiatorMechanismInterface
 from negmas.negotiators import Negotiator
 from negmas.outcomes import Issue
-from negmas.situated import Agent
-from negmas.situated import Breach
-from negmas.situated import Contract
+from negmas.situated import Agent, Breach, Contract
 
-from .common import CFP
-from .common import FinancialReport
-from .common import Loan
-from .common import ManufacturingProfileCompiled
-from .common import Process
-from .common import Product
-from .common import ProductManufacturingInfo
+from .common import (
+    CFP,
+    FinancialReport,
+    Loan,
+    ManufacturingProfileCompiled,
+    Process,
+    Product,
+    ProductManufacturingInfo,
+)
 
 if TYPE_CHECKING:
     from .awi import SCMLAWI
@@ -34,7 +30,9 @@ __all__ = ["SCML2019Agent"]
 class SCML2019Agent(Agent):
     """The base for all SCM Agents"""
 
-    def __init__(self, name: str = None, ufun: "UtilityFunction" = None):
+    def __init__(
+        self, name: Optional[str] = None, ufun: Optional["UtilityFunction"] = None
+    ):
         super().__init__(name=name, ufun=ufun)
         self.line_profiles: Dict[int, ManufacturingProfileCompiled] = {}
         """A mapping specifying for each `Line` index, all the profiles used to run it in the factory"""
