@@ -1,30 +1,25 @@
 import math
-from abc import ABC
-from abc import abstractmethod
-from dataclasses import dataclass
-from dataclasses import field
-from typing import Any
-from typing import Callable
-from typing import Collection
-from typing import Dict
-from typing import List
-from typing import Optional
+from abc import ABC, abstractmethod
+from dataclasses import dataclass, field
+from typing import Any, Callable, Collection, Dict, List, Optional
 
 import numpy as np
 from negmas.situated import Contract
 
 from scml.scml2019.helpers import zero_runs
 
-from .common import NO_PRODUCTION
-from .common import Job
-from .common import ManufacturingProfileCompiled
-from .common import Process
-from .common import Product
-from .common import ProductionNeed
-from .common import ProductManufacturingInfo
-from .common import SCMLAgreement
-from .simulators import FactorySimulator
-from .simulators import transaction
+from .common import (
+    NO_PRODUCTION,
+    Job,
+    ManufacturingProfileCompiled,
+    Process,
+    Product,
+    ProductionNeed,
+    ProductManufacturingInfo,
+    SCMLAgreement,
+)
+from .awi import SCMLAWI
+from .simulators import FactorySimulator, transaction
 
 __all__ = ["ScheduleInfo", "Scheduler", "GreedyScheduler"]
 
@@ -103,7 +98,7 @@ class Scheduler(ABC):
         self.horizon = horizon
         self.n_steps = 0
         self.n_lines = 0
-        self.simulator: FactorySimulator = None
+        self.simulator: FactorySimulator | None = None
         self.products: List[Product] = []
         self.processes: List[Process] = []
         self.profiles: List[ManufacturingProfileCompiled] = []
@@ -587,7 +582,6 @@ class GreedyScheduler(Scheduler):
                             continue  # should never hit this
                         jobs.append(job)
                         # find the needs
-                        new_needs = []
                         process = self.processes[process_index]
                         length = profile.n_steps
                         for i in process.inputs:
@@ -727,7 +721,6 @@ class GreedyScheduler(Scheduler):
         ensure_storage_for: int = 0,
         start_at: int = 0,
     ):
-
         # Now, schedule the contracts
         schedule = self.schedule_contracts(
             contracts=contracts,
