@@ -1,28 +1,27 @@
 """Implements the world class for the SCML2020 world """
-from typing import Any
-from typing import Dict
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from collections import defaultdict
+from typing import Any, Dict, List, Literal, Optional, Tuple, Union
 
 import numpy as np
-from negmas import Agent
-from negmas import Breach
-from negmas import Contract
-from negmas import Issue
-from negmas import MechanismState
-from negmas import Negotiator
-from negmas import NegotiatorMechanismInterface
-from negmas import RenegotiationRequest
-from negmas import make_issue
-from negmas.helpers import get_full_type_name
-from negmas.helpers import instantiate
+from negmas import (
+    Agent,
+    Breach,
+    Contract,
+    DiscreteCartesianOutcomeSpace,
+    Issue,
+    MechanismState,
+    Negotiator,
+    NegotiatorMechanismInterface,
+    RenegotiationRequest,
+    make_issue,
+    make_os,
+)
+from negmas.helpers import get_full_type_name, instantiate
 from negmas.situated import Adapter
 
+from .common import Failure
 from ..oneshot.agent import OneShotAgent
-from ..oneshot.common import OneShotProfile
-from ..oneshot.common import OneShotState
+from ..oneshot.common import OneShotProfile, OneShotState
 from ..oneshot.mixins import OneShotUFunCreatorMixin
 from .components.production import DemandDrivenProductionStrategy
 from .components.signing import SignAll
@@ -60,14 +59,6 @@ class SCML2020Agent(Agent):
             "levels": self.awi.my_input_products if self.awi else None,
         }
 
-    def on_contract_breached(
-        self, contract: Contract, breaches: List[Breach], resolution: Optional[Contract]
-    ) -> None:
-        pass
-
-    def on_contract_executed(self, contract: Contract) -> None:
-        pass
-
     def _respond_to_negotiation_request(
         self,
         initiator: str,
@@ -84,6 +75,14 @@ class SCML2020Agent(Agent):
             initiator, issues, annotation, mechanism
         )
 
+    def on_contract_breached(
+        self, contract: Contract, breaches: List[Breach], resolution: Optional[Contract]
+    ) -> None:
+        pass
+
+    def on_contract_executed(self, contract: Contract) -> None:
+        pass
+
     def set_renegotiation_agenda(
         self, contract: Contract, breaches: List[Breach]
     ) -> Optional[RenegotiationRequest]:
@@ -279,6 +278,51 @@ class AWIHelper:
         self._owner = owner
         self._world = owner.awi._world
 
+    @property
+    def sales(self) -> dict[str, int]:
+        """Sales per customer so far (this day)"""
+        return defaultdict(int)
+
+    @property
+    def supplies(self) -> dict[str, int]:
+        """Supplies per supplier so far (this day)"""
+        return defaultdict(int)
+
+    @property
+    def total_sales(self) -> int:
+        """Total sales so far (this day)"""
+        return 0
+
+    @property
+    def total_supplies(self) -> int:
+        """Total supplies so far (this day)"""
+        return 0
+
+    @property
+    def needed_sales(self) -> int:
+        """Sales that need to be secured (exogenous input + total supplies - exogenous output - total sales so far)"""
+        return 0
+
+    @property
+    def needed_supplies(self) -> int:
+        """Supplies that need to be secured (exogenous output + total sales - exogenous input - total supplies so far)"""
+        return 0
+
+    @property
+    def current_inventory(self) -> tuple[int, int]:
+        """Current input and output inventory quantity"""
+        return (0, 0)
+
+    @property
+    def current_inventory_input(self) -> int:
+        """Current input inventory quantity"""
+        return 0
+
+    @property
+    def current_inventory_output(self) -> int:
+        """Current output inventory quantity"""
+        return 0
+
     # private information
     # ===================
     def penalty_multiplier(self, is_input: bool, unit_price: float) -> float:
@@ -351,10 +395,14 @@ class AWIHelper:
         """Gets the profile (static private information) associated with the agent"""
         return self._owner.get_profile()
 
-    def state(self) -> Any:
+    def state(self) -> OneShotState:
         q, p = self._owner.get_exogenous_input()
         qo, po = self._owner.get_exogenous_output()
+        # TODO add missing components
+        all_agents = [_ for _ in self._world.agents.keys() if self.is_system(_)]
         return OneShotState(
+            production_capacities=[],
+            perishable=True,
             exogenous_input_quantity=q,
             exogenous_input_price=p,
             exogenous_output_quantity=qo,
@@ -362,8 +410,64 @@ class AWIHelper:
             disposal_cost=self._owner.get_disposal_cost(),
             shortfall_penalty=self._owner.get_shortfall_penalty(),
             current_balance=self._owner.get_current_balance(),
+            total_sales=0,
+            total_supplies=0,
+            total_future_sales=0,
+            total_future_supplies=0,
+            n_products=self.n_products,
+            n_processes=self.n_processes,
+            n_competitors=self.n_competitors,
+            all_suppliers=self.all_suppliers,
+            all_consumers=self.all_consumers,
+            catalog_prices=self.catalog_prices.tolist(),
+            price_multiplier=1.0,
+            is_exogenous_forced=False,
+            current_step=self._owner.awi.current_step,
+            n_steps=self._owner.awi.n_steps,
+            relative_simulation_time=self._owner.awi.relative_time,
+            profile=self.profile,
+            n_lines=self.n_lines,
+            is_first_level=self.is_first_level,
+            is_last_level=self.is_last_level,
+            is_middle_level=self.is_middle_level,
+            my_input_product=self.my_input_product,
+            my_output_product=self.my_output_product,
+            level=self.level,
+            my_suppliers=self.my_suppliers,
+            my_consumers=self.my_consumers,
+            my_partners=self.my_partners,
+            penalties_scale=self.penalties_scale,  # type: ignore
+            n_input_negotiations=self.n_input_negotiations,
+            n_output_negotiations=self.n_output_negotiations,
+            trading_prices=self.trading_prices.tolist(),
+            exogenous_contract_summary=self.exogenous_contract_summary,
+            current_input_outcome_space=self.current_input_outcome_space,
+            current_output_outcome_space=self.current_output_outcome_space,
+            current_negotiation_details=dict(),
+            sales=dict(),
+            supplies=dict(),
+            needed_sales=0,
+            needed_supplies=0,
+            bankrupt_agents=[_ for _ in all_agents if self.is_bankrupt(_)],
+            reports_of_agents=dict(
+                zip(
+                    all_agents,
+                    [self.reports_of_agent(_) for _ in all_agents],
+                )
+            ),
+            # running_negotiations=dict(),
         )
 
+    @property
+    def is_perishable(self) -> bool:
+        """Are all products perishable (original design of OneShot)"""
+        return False
+
+    @property
+    def current_storage_cost(self) -> float:
+        """Cost of storing one unit (penalizes buying too much/ selling too little)"""
+        return 0.0
+
     @property
     def price_multiplier(self):
         return self._owner.price_multiplier
@@ -417,7 +521,7 @@ class AWIHelper:
                 make_issue(values=t, name="time"),
                 make_issue(values=u, name="unit_price"),
             ]
-        return issues
+        return issues  # type: ignore
 
     @property
     def current_output_issues(self) -> List[Issue]:
@@ -430,10 +534,20 @@ class AWIHelper:
                 make_issue(values=t, name="time"),
                 make_issue(values=u, name="unit_price"),
             ]
-        return issues
+        return issues  # type: ignore
 
     @property
-    def penalties_scale(self) -> str:
+    def current_input_outcome_space(self) -> DiscreteCartesianOutcomeSpace:
+        return make_os(self.current_input_issues) if self.current_input_issues else None  # type: ignore
+
+    @property
+    def current_output_outcome_space(self) -> DiscreteCartesianOutcomeSpace:
+        return (  # type: ignore
+            make_os(self.current_output_issues) if self.current_output_issues else None
+        )
+
+    @property
+    def penalties_scale(self) -> Literal["trading", "catalog", "input", "unit", "none"]:
         return "unit"
 
     # Public Information
@@ -449,7 +563,7 @@ class AWIHelper:
             step.
         """
         if not self._world.publish_exogenous_summary:
-            return None
+            return []
         n_steps = self._owner.awi.n_steps
         n_products = self._owner.awi.n_products
         y = self._world.exogenous_contracts_summary
@@ -463,6 +577,13 @@ class AWIHelper:
         ]
         return summary
 
+    @property
+    def allow_zero_quantity(self) -> bool:
+        """
+        Does negotiations allow zero quantity?
+        """
+        return False
+
     # Everything else
     # ===============
     def __getattr__(self, attr):
@@ -492,7 +613,7 @@ class OneShotAdapter(
         ufun=None,
     ):
         if obj:
-            self.oneshot_type = get_full_type_name(obj)
+            self.oneshot_type = get_full_type_name(obj)  # type: ignore
             # note that this may not be true and we cannot guarantee that
             # we can instantiate an agent of the same type
             self.oneshot_params = dict()
@@ -500,17 +621,18 @@ class OneShotAdapter(
             if not oneshot_params:
                 oneshot_params = dict()
             self.oneshot_type, self.oneshot_params = (
-                get_full_type_name(oneshot_type),
+                get_full_type_name(oneshot_type),  # type: ignore
                 oneshot_params,
             )
-            obj = instantiate(oneshot_type, **oneshot_params)
+            obj = instantiate(oneshot_type, **oneshot_params)  # type: ignore
         super().__init__(obj=obj, name=name, type_postfix=type_postfix, ufun=ufun)
-        obj.connect_to_2021_adapter(self)
+        obj.connect_to_2021_adapter(self)  # type: ignore
+        self._obj: SCML2020Agent
 
     def init(self):
         self._oneshot_awi = AWIHelper(self)
-        self._obj._awi = self._oneshot_awi
-        self._obj.ufun = self.make_ufun(add_exogenous=True)
+        self._obj._awi = self._oneshot_awi  # type: ignore
+        self._obj.ufun = self.make_ufun(add_exogenous=True)  # type: ignore
         super().init()
         self._obj.init()
 
@@ -584,8 +706,8 @@ class OneShotAdapter(
         if hasattr(self._obj, "before_step"):
             self._obj.before_step()
 
-    def make_ufun(self, add_exogenous: bool):
-        return super().make_ufun(add_exogenous, in_adapter=True)
+    def make_ufun(self, add_exogenous: bool):  # type: ignore
+        return super().make_ufun(add_exogenous, in_adapter=True)  # type: ignore
 
     def to_dict(self):
         return {
@@ -610,7 +732,7 @@ class OneShotAdapter(
         #     or issues[TIME].max_value != self.awi.current_step + 1
         # ):
         #     return None
-        return self._obj.create_negotiator()
+        return self._obj.create_negotiator()  # type: ignore
 
     def get_disposal_cost(self) -> float:
         # penalty for buying too much
@@ -628,6 +750,12 @@ class OneShotAdapter(
     def get_disposal_cost_dev(self):
         return 0.0
 
+    def get_storage_cost_mean(self):
+        return 0.0
+
+    def get_storage_cost_dev(self):
+        return 0.0
+
     def get_profile(self):
         return OneShotProfile(
             cost=float(self.awi.profile.costs[:, self.awi.my_input_product].mean()),
@@ -637,6 +765,8 @@ class OneShotAdapter(
             disposal_cost_mean=self.get_disposal_cost_mean(),
             shortfall_penalty_dev=self.get_shortfall_penalty_dev(),
             disposal_cost_dev=self.get_disposal_cost_dev(),
+            storage_cost_mean=self.get_storage_cost_mean(),
+            storage_cost_dev=self.get_storage_cost_dev(),
         )
 
     def get_shortfall_penalty(self):
@@ -651,3 +781,30 @@ class OneShotAdapter(
 
     def get_exogenous_input(self) -> Tuple[int, int]:
         return 0, 0
+
+    @property
+    def is_perishable(self) -> bool:
+        """Are all products perishable (original design of OneShot)"""
+        return False
+
+    @property
+    def current_disposal_cost(self) -> float:
+        """Cost of storing one unit (penalizes buying too much/ selling too little)"""
+        return 0.0
+
+    @property
+    def current_storage_cost(self) -> float:
+        """Cost of storing one unit (penalizes buying too much/ selling too little)"""
+        return 0.0
+
+    @property
+    def current_shortfall_penalty(self) -> float:
+        """Cost of failure to deliver one unit (penalizes buying too little / selling too much)"""
+        return 0.0
+
+    @property
+    def allow_zero_quantity(self) -> bool:
+        """
+        Does negotiations allow zero quantity?
+        """
+        return False
