@@ -2,10 +2,11 @@
 Implements the `DecentralizingAgent` which creates ony buy and one sell controller for each time-step and relinquishes
 control of negotiations to buy/sell the required number of items of its input/output product.
 """
-from scml.scml2020.components import MovingRangeNegotiationManager
-from scml.scml2020.components import SupplyDrivenProductionStrategy
+from scml.scml2020.components import (
+    MovingRangeNegotiationManager,
+    SupplyDrivenProductionStrategy,
+)
 
-from ..components import KeepOnlyGoodPrices
 from ..components.prediction import MarketAwareTradePredictionStrategy
 from ..components.trading import PredictionBasedTradingStrategy
 from ..world import SCML2020Agent
@@ -28,5 +29,5 @@ class MarketAwareMovingRangeAgent(MarketAwareTradePredictionStrategy, MovingRang
             *args,
             min_price_margin=min_price_margin,
             max_price_margin=max_price_margin,
-            **kwargs
+            **kwargs,
         )
