@@ -1,8 +1,6 @@
 """Implements a randomly behaving agent"""
 
-from negmas import Issue
 from negmas import RandomUtilityFunction
-from negmas.outcomes.issue_ops import enumerate_issues
 
 from .indneg import IndependentNegotiationsAgent
 
