@@ -2,29 +2,19 @@
 import math
 import random
 from collections import defaultdict
-from typing import Any
-from typing import Dict
-from typing import List
 from typing import Optional
 
 import numpy as np
-from negmas import Contract
-from negmas import Issue
-from negmas import MechanismState
-from negmas import Negotiator
-from negmas import NegotiatorMechanismInterface
-from negmas import Outcome
-from negmas import ResponseType
-from negmas.sao import SAONMI
-from negmas.sao import SAONegotiator
-from negmas.sao import SAOState
+from negmas import (
+    MechanismState,
+    Outcome,
+    ResponseType,
+)
+from negmas.sao import SAONMI, SAONegotiator, SAOState
 
-from scml.scml2020 import AWI
-from scml.scml2020 import NO_COMMAND
-from scml.scml2020 import QUANTITY
-from scml.scml2020 import TIME
-from scml.scml2020 import UNIT_PRICE
-from scml.scml2020 import SCML2020Agent
+from scml.scml2020.agent import SCML2020Agent
+from scml.scml2020.awi import AWI
+from scml.scml2020.common import NO_COMMAND, QUANTITY, TIME, UNIT_PRICE
 
 __all__ = ["SatisficerAgent"]
 
@@ -59,11 +49,9 @@ class ObedientNegotiator(SAONegotiator):
         """Simply calls the corresponding method on the owner"""
         return self.owner.propose(state, self.nmi, self.is_selling, self.is_requested)
 
-    def respond(self, state: MechanismState, offer: Outcome) -> ResponseType:
+    def respond(self, state: MechanismState, source: str = "") -> ResponseType:
         """Simply calls the corresponding method on the owner"""
-        return self.owner.respond(
-            state, self.nmi, offer, self.is_selling, self.is_requested
-        )
+        return self.owner.respond(state, self.nmi, self.is_selling, self.is_requested)
 
 
 class SatisficerAgent(SCML2020Agent):
@@ -176,7 +164,7 @@ class SatisficerAgent(SCML2020Agent):
         first_supply = max(s, level)
         last_sale = steps - (processes - level - 1)
         first_sale = first_supply + 1
-        last_supply = last_sale - 1
+        last_sale - 1
 
         period = last_sale - first_sale
 
@@ -417,7 +405,6 @@ class SatisficerAgent(SCML2020Agent):
 
     def on_contracts_finalized(self, signed, cancelled, rejectors):
         # Updates secured_sales/supplies
-        awi: AWI = self.awi
         sell_contracts = [_ for _ in signed if _.annotation["seller"] == self.id]
         buy_contracts = [_ for _ in signed if _.annotation["seller"] != self.id]
 
@@ -445,7 +432,7 @@ class SatisficerAgent(SCML2020Agent):
 
     def do_production(self) -> int:
         # Produce everything I can
-        awi: AWI = self.awi
+        awi: AWI = self.awi  # type: ignore
         commands = NO_COMMAND * np.ones(awi.n_lines, dtype=int)
 
         # find how much input material do we have
@@ -592,7 +579,7 @@ class SatisficerAgent(SCML2020Agent):
 
         return tuple(offer)
 
-    def respond(self, state, ami, offer, is_selling, is_requested):
+    def respond(self, state, ami, is_selling, is_requested):
         """
         Responds to an offer from one partner.
 
@@ -614,6 +601,7 @@ class SatisficerAgent(SCML2020Agent):
               the trading price for buying/selling.
 
         """
+        offer = state.current_offer
         # Find the price range for this negotiation
         p0, p1 = ami.issues[UNIT_PRICE].min_value, ami.issues[UNIT_PRICE].max_value
 
