@@ -1,28 +1,26 @@
 """
 Implements the Agent-World-Interface for SCML2020 worlds
 """
-from typing import Any
-from typing import Dict
-from typing import Iterable
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
-from negmas import AgentWorldInterface
-from negmas import ControlledSAONegotiator
-from negmas import Negotiator
-from negmas import SAOController
-from negmas import SAONegotiator
-from negmas import make_issue
+from negmas import (
+    AgentWorldInterface,
+    ControlledSAONegotiator,
+    Negotiator,
+    SAOController,
+    SAONegotiator,
+    make_issue,
+)
 
-from .common import ANY_LINE
-from .common import ANY_STEP
-from .common import FactoryProfile
-from .common import FactoryState
-from .common import FinancialReport
-from .common import is_system_agent
+from .common import (
+    ANY_LINE,
+    ANY_STEP,
+    FactoryProfile,
+    FactoryState,
+    FinancialReport,
+    is_system_agent,
+)
 
 __all__ = [
     "AWI",
@@ -518,6 +516,13 @@ class AWI(AgentWorldInterface):
             else None
         )
 
+    @property
+    def allow_zero_quantity(self) -> bool:
+        """
+        Does negotiations allow zero quantity?
+        """
+        return False
+
     @property
     def state(self) -> FactoryState:
         """Receives the factory state"""
@@ -545,7 +550,7 @@ class AWI(AgentWorldInterface):
         steps = sorted(
             int(i) for i in self.bb_query("reports_time", None, query_keys=True).keys()
         )
-        for (s, prev) in zip(steps[1:], steps[:-1]):
+        for s, prev in zip(steps[1:], steps[:-1]):
             if s > step:
                 return self.bb_read("reports_time", prev)
         return self.bb_read("reports_time", str(steps[-1]))
@@ -566,11 +571,6 @@ class AWI(AgentWorldInterface):
         """Returns a list of agent IDs for all consumers for every product"""
         return self._world.consumers
 
-    @property
-    def catalog_prices(self) -> np.ndarray:
-        """Returns the catalog prices of all products"""
-        return self._world.catalog_prices
-
     @property
     def inputs(self) -> np.ndarray:
         """Returns the number of inputs to every production process"""
@@ -581,16 +581,6 @@ class AWI(AgentWorldInterface):
         """Returns the number of outputs to every production process"""
         return self._world.process_outputs
 
-    @property
-    def n_products(self) -> int:
-        """Returns the number of products in the system"""
-        return len(self._world.catalog_prices)
-
-    @property
-    def n_processes(self) -> int:
-        """Returns the number of processes in the system"""
-        return self.n_products - 1
-
     @property
     def n_competitors(self) -> int:
         """Returns the number of factories/agents in the same production level"""
@@ -651,6 +641,11 @@ class AWI(AgentWorldInterface):
         """The number of lines in the corresponding factory. You can read `state` to get this among other information"""
         return self.state.n_lines
 
+    @property
+    def catalog_prices(self) -> np.ndarray:
+        """Returns the catalog prices of all products"""
+        return self._world.catalog_prices
+
     @property
     def n_products(self) -> int:
         """Number of products in the world"""
@@ -658,8 +653,8 @@ class AWI(AgentWorldInterface):
 
     @property
     def n_processes(self) -> int:
-        """Number of processes in the world"""
-        return self.state.n_processes
+        """Returns the number of processes in the system"""
+        return self.n_products - 1
 
     @property
     def is_first_level(self):
