@@ -1,3 +1,14 @@
+from __future__ import annotations
+from typing import TYPE_CHECKING
+
+import numpy as np
+from negmas import Contract
+
+from scml.scml2020.common import NO_COMMAND
+
+if TYPE_CHECKING:
+    from scml.scml2020.agent import SCML2020Agent
+
 __all__ = [
     "ProductionStrategy",
     "SupplyDrivenProductionStrategy",
@@ -5,15 +16,6 @@ __all__ = [
     "TradeDrivenProductionStrategy",
 ]
 
-from typing import Dict
-from typing import List
-from typing import Tuple
-
-import numpy as np
-from negmas import Contract
-
-from scml.scml2020.common import NO_COMMAND
-
 
 class ProductionStrategy:
     """Represents a strategy for controlling production.
@@ -49,7 +51,7 @@ class ProductionStrategy:
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.schedule_range: Dict[str, Tuple[int, int, bool]] = dict()
+        self.schedule_range: dict[str, tuple[int, int, bool]] = dict()
         """Gives the range of steps at which the production needed for a given contract are scheduled"""
 
     def can_be_produced(self, contract_id: str):
@@ -114,10 +116,10 @@ class SupplyDrivenProductionStrategy(ProductionStrategy):
         self.awi.set_commands(commands)
 
     def on_contracts_finalized(
-        self: "SCML2020Agent",
-        signed: List[Contract],
-        cancelled: List[Contract],
-        rejectors: List[List[str]],
+        self: SCML2020Agent,
+        signed: list[Contract],
+        cancelled: list[Contract],
+        rejectors: list[list[str]],
     ) -> None:
         super().on_contracts_finalized(signed, cancelled, rejectors)
         latest = self.awi.n_steps - 2
@@ -171,10 +173,10 @@ class DemandDrivenProductionStrategy(ProductionStrategy):
     """
 
     def on_contracts_finalized(
-        self: "SCML2020Agent",
-        signed: List[Contract],
-        cancelled: List[Contract],
-        rejectors: List[List[str]],
+        self: SCML2020Agent,
+        signed: list[Contract],
+        cancelled: list[Contract],
+        rejectors: list[list[str]],
     ) -> None:
         super().on_contracts_finalized(signed, cancelled, rejectors)
         for contract in signed:
@@ -228,10 +230,10 @@ class TradeDrivenProductionStrategy(ProductionStrategy):
     """
 
     def on_contracts_finalized(
-        self: "SCML2020Agent",
-        signed: List[Contract],
-        cancelled: List[Contract],
-        rejectors: List[List[str]],
+        self: SCML2020Agent,
+        signed: list[Contract],
+        cancelled: list[Contract],
+        rejectors: list[list[str]],
     ) -> None:
         super().on_contracts_finalized(signed, cancelled, rejectors)
         for contract in signed:
