@@ -1,15 +1,15 @@
-from typing import List
-from typing import Optional
+from typing import List, Optional
 
 import numpy as np
 from negmas import Contract
 
-from scml.scml2020.common import ANY_LINE
-from scml.scml2020.common import is_system_agent
+from scml.scml2020.common import ANY_LINE, is_system_agent
 from scml.scml2020.components import SignAllPossible
-from scml.scml2020.components.prediction import FixedTradePredictionStrategy
-from scml.scml2020.components.prediction import MarketAwareTradePredictionStrategy
-from scml.scml2020.components.prediction import MeanERPStrategy
+from scml.scml2020.components.prediction import (
+    FixedTradePredictionStrategy,
+    MarketAwareTradePredictionStrategy,
+    MeanERPStrategy,
+)
 
 __all__ = [
     "TradingStrategy",
@@ -369,7 +369,7 @@ class PredictionBasedTradingStrategy(
         s = self.awi.current_step
         for contract, indx in contracts:
             is_seller = contract.annotation["seller"] == self.id
-            q, u, t = (
+            q, _u, t = (
                 contract.agreement["quantity"],
                 contract.agreement["unit_price"],
                 contract.agreement["time"],
@@ -414,7 +414,7 @@ class PredictionBasedTradingStrategy(
 
     def _format(self, c: Contract):
         return (
-            f"{f'>' if c.annotation['seller'] == self.id else '<'}"
+            f"{'>' if c.annotation['seller'] == self.id else '<'}"
             f"{c.annotation['buyer'] if c.annotation['seller'] == self.id else c.annotation['seller']}: "
             f"{c.agreement['quantity']} of {c.annotation['product']} @ {c.agreement['unit_price']} on {c.agreement['time']}"
         )
