@@ -1,40 +1,45 @@
 import random
 from collections import defaultdict
-from typing import Any
-from typing import Callable
-from typing import Dict
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Type
-from typing import Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    TYPE_CHECKING,
+)
 
 import numpy as np
-from negmas import AgentWorldInterface
-from negmas import ControlledNegotiator
-from negmas import LinearUtilityFunction
-from negmas import MechanismState
-from negmas import Outcome
-from negmas import PolyAspiration
-from negmas import ResponseType
-from negmas import UtilityFunction
-from negmas import make_issue
-from negmas import make_os
-from negmas import outcome_is_valid
+from negmas import (
+    AgentWorldInterface,
+    ControlledNegotiator,
+    LinearUtilityFunction,
+    MechanismState,
+    Outcome,
+    ResponseType,
+    make_issue,
+    outcome_is_valid,
+    Preferences,
+    UtilityFunction,
+)
 from negmas.common import NegotiatorMechanismInterface
-from negmas.events import Notification
-from negmas.events import Notifier
+from negmas.events import Notification, Notifier
 from negmas.helpers import instantiate
-from negmas.sao import SAOController
-from negmas.sao import SAONegotiator
-from negmas.sao import SAOResponse
-from negmas.sao import SAOState
-from negmas.sao import SAOSyncController
-from negmas.sao.negotiators.controlled import ControlledSAONegotiator
+from negmas.sao import (
+    SAOController,
+    SAONegotiator,
+    SAOResponse,
+    SAOState,
+    SAOSyncController,
+)
 
-from scml.scml2020.common import QUANTITY
-from scml.scml2020.common import TIME
-from scml.scml2020.common import UNIT_PRICE
+if TYPE_CHECKING:
+    from scml.scml2020.components.trading import PredictionBasedTradingStrategy
+
+from scml.scml2020.common import QUANTITY, TIME, UNIT_PRICE
 
 __all__ = ["StepController", "SyncController"]
 
@@ -143,12 +148,15 @@ class StepController(SAOController, Notifier):
             return None
         try:
             self.__negotiator._nmi = self.negotiators[negotiator_id][0].nmi
-        except:
+        except Exception:
             self.__negotiator._nmi = None
         return self.__negotiator.propose(state)
 
     def respond(
-        self, negotiator_id: str, state: MechanismState, offer: "Outcome"
+        self,
+        negotiator_id: str,
+        state: MechanismState,
+        source: str = "",
     ) -> ResponseType:
         if negotiator_id not in self.negotiators.keys():
             return ResponseType.END_NEGOTIATION
@@ -157,7 +165,10 @@ class StepController(SAOController, Notifier):
         # if negotiator_id not in self.negotiators:
         #     breakpoint()
         self.__negotiator._nmi = self.negotiators[negotiator_id][0].nmi
-        return self.__negotiator.respond(offer=offer, state=state)
+        try:
+            return self.__negotiator.respond(state=state, source=source)
+        except TypeError:
+            return self.__negotiator.respond(state=state)
 
     def __str__(self):
         return (
